from typing import Optional

from jenkinsctl.configs.session import Session
from jenkinsctl.jenkins.commons import get_last_build_no_if_none
from jenkinsctl.jenkins.job import get_build, build_job
from jenkinsctl.jenkins.utils import get_build_params, normalize_job_path


def rebuild_handler(session: Session, job_name: str, build_no: Optional[int]):
    job_name = normalize_job_path(job_name)
    build_no = get_last_build_no_if_none(session, job_name, build_no)
    build = get_build(session, job_name, build_no)
    location = build_job(session, job_name, get_build_params(build))
    print(location)



from typing import Optional

from jenkinsctl.configs.session import Session
from jenkinsctl.jenkins.commons import get_last_build_no_if_none
from jenkinsctl.jenkins.console_util import print_json
from jenkinsctl.jenkins.job import progressive_log, get_build
from jenkinsctl.jenkins.utils import normalize_job_path


def json_handler(session: Session, job_name: str, build_no: Optional[int]):
    job_name = normalize_job_path(job_name)
    build_no = get_last_build_no_if_none(session, job_name, build_no)
    build = get_build(session, job_name, build_no)
    print_json(build)


import logging
from contextlib import contextmanager

from jenkinsctl.configs.config import settings
from jenkinsctl.configs.session import Session

log = logging.getLogger(__name__)

@contextmanager
def error_handler_and_session():
    server_url: str = settings.server_url
    username: str = settings.username
    api_key: str = settings.api_key

    session = None
    try:
        session = Session(server_url)
        session.auth = (username, api_key)
        yield session
    except Exception as e:
        log.debug(f"An error occurred", exc_info=e )
        print(f"An error occurred, run same command with --verbose flag to print error details")
    finally:
        if session:
            session.close()


import time

from urllib.parse import urlparse

from jenkinsctl.configs.session import Session

def _remove_base_url(url: str):
    return urlparse(url).path

def _get(session: Session, url: str):
    url = _remove_base_url(url)
    url = f"{url}api/json"
    return session.get(url).json()


def build_job(session: Session, job_name: str, params: dict):
    response = None
    if len(params) == 0:
        url = f"/job/{job_name}/build"
        response = session.post(url)
    else:
        url = f"/job/{job_name}/buildWithParameters"
        response = session.post(url, params=params)

    return response.headers.get("Location")


def get_job(session: Session, job_name: str):
    url = f"/job/{job_name}/"
    return _get(session, url)


def get_jobs(session: Session, folder_name: str):
    if folder_name.strip() == "":
        return _get(session, "")

    url = f"/job/{folder_name}/"
    return _get(session, url)


def get_builds_iter(session: Session, job_json):
    builds = job_json["builds"]
    for build in builds:
        yield _get_build(session, job_json, build["number"])


def _get_build(session: Session, job_json, build_no):
    builds = job_json["builds"]
    build = next((build for build in builds if build["number"] == build_no), None)

    return _get(session, build["url"])


def get_build(session: Session, job_name: str, build_no: int):
    url = f"/job/{job_name}/{build_no}/"
    return _get(session, url)


def progressive_log(session: Session, job_name: str, build_no: int):
    url = f"/job/{job_name}/{build_no}/logText/progressiveText"
    start_byte = 0
    while True:
        response = session.get(url, params={'start': start_byte})
        text = response.text
        print(text, end="")
        start_byte = int(response.headers.get('X-Text-Size', 0))
        if response.headers.get('X-More-Data') == 'false' or text.strip() == "":
            break
        time.sleep(2)



from urllib.parse import urljoin

import requests
import logging

logger = logging.getLogger(__name__)

class Session(requests.Session):
    def __init__(self, base_url: str):
        super().__init__()
        self.base_url = base_url

    def request(
            self,
            method,
            url,
            **kwargs
    ):
        joined_url = urljoin(self.base_url, url)
        return super().request(method, joined_url, **kwargs)




import logging
import sys
from io import TextIOWrapper
from typing import List

import yaml

from jenkinsctl.configs.session import Session
from jenkinsctl.jenkins.console_util import json_preety
from jenkinsctl.jenkins.job import build_job

log = logging.getLogger(__name__)


def get_config_from_yaml(file):
    config_data = None
    try:
        with open(file.name, "r") as config_file:
            config_data = yaml.safe_load(config_file)  # Use safe_load for YAML
    except FileNotFoundError:
        print(f"Error: Configuration file '{file}' not found.")
        sys.exit()
    except yaml.YAMLError as e:
        print(f"Error: Invalid YAML format in '{file}': {e}")
        sys.exit()

    return config_data


def get_conf(file, params):
    config = get_config_from_yaml(file)
    override_params(params, config)
    return config


def override_params(params, file_config):
    for param in params:
        name, value = param.split('=')
        file_config['params'][name] = value


def build_handler(session: Session, file: TextIOWrapper, params: List[str]):
    conf = get_conf(file, params)
    log.debug(f"config: {json_preety(conf)}")
    location = build_job(session, conf["job"], conf["params"])
    print(location)
